#!/bin/bash
#
# Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of le-server
#
# le-server is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

TMP_DIR="$(mktemp -d --tmpdir="/tmp" ecd_XXXXX)"
INSTALLING="$TMP_DIR/installing"

# Regex to match all packages with educational content
MEC_REGEX="^mec-(aula|casa|dp|museu|vol1|vol2)"

# We are assuming that this script will be started at night and it is supposed
# to run until TIME_LIMIT, which is some time in the morning. For example,
# cron starts it at 23 o'clock, so it will run until TIME_LIMIT of the next day
TIME_LIMIT="06" #am

# If the program isn't running as root, it doesn't have privileges to
# use apt functions and can't work, so we exit with an error code
if [ "$(id -u)" -ne 0 ]; then
   exit 1
fi

# This functions is necessary to guarantee that the program will quit before
# TIME_LIMIT. This way we don't harm the classes using the whole bandwidth
# of the school
function monitor_time() {
    local pid_to_kill=$1
    while true; do
        # If the main program completes its downloads and stop running,
        # it's not necessary to test it anymore, then we quit
        if ! kill -0 $pid_to_kill; then
            exit 0
        fi
        # If we have reached the time limit and we are not currently
        # installing some package, we kill the main program and quit
        if [ "$(date +%H)" -eq "$TIME_LIMIT" -a ! -f $INSTALLING ]; then
            kill $pid_to_kill
            rm -r $TMP_DIR
            exit 0
        fi
        # We don't want to sleep to much, otherwise the main program could
        # download more packages and start to install them again, and again,
        # ... , and the classes in the school would have started and we would
        # still be downloading more stuff, which is not cool.
        sleep 60
    done
}

# Retrieve all available packages with educational content
available_pkgs="$(apt-cache search -n "$MEC_REGEX" | awk '{print $1}')"

# Open the function to monitor the time spent downloading/installing
# educational content
monitor_time $$ &

for pkg in $available_pkgs; do
    if ! dpkg --get-selections | grep -w "${pkg}"; then
        # Download first, install later
        apt-get -d install $pkg

        # Sign that apt is installing something to prevent this program from
        # killing apt in the middle of the process
        touch $INSTALLING
        apt-get install -y $pkg
        rm -f $INSTALLING
    fi
done

rm -r $TMP_DIR
